@@ -1,34 +1,27 @@
 \dontrun{#REX
 library(psd)
+
 ##
 ## Spectral properties from the number of tapers used
 ## (portions extracted from overview vignette)
 ##
 
-##
-## Theoretical uncertainties from Chi^2 distribution
-##
+#
+# Theoretical uncertainties from Chi^2 distribution
+#
 sp <- spectral_properties(as.tapers(1:50), p=0.95, db.ci=TRUE)
 par(las=1)
 plot(stderr.chi.upper ~ taper, sp, type="s",
        ylim=c(-10,20), yaxs="i", xaxs="i",
        xlab=expression("number of tapers ("* nu/2 *")"), ylab="dB",
        main="Spectral uncertainties")
-mtext("(additive factor)", line=.3)
 lines(stderr.chi.lower ~ taper, sp, type="s")
 lines(stderr.chi.median ~ taper, sp, type="s", lwd=2)
 lines(stderr.chi.approx ~ taper, sp, type="s", col="red",lwd=2)
-# to reach 3 db width confidence interval at p=.95
-abline(v=33, lty=3)
-legend("topright",
-        c(expression("Based on "* chi^2 *"(p,"*nu*") and (1-p,"*nu*")"),
-          expression(""* chi^2 *"(p=0.5,"*nu*")"),
-          "approximation"),
-lwd=c(1,3,3), col=c("black","black","red"), bg="white")
 
-##
-## An example using the Project MAGNET dataset
-##
+#
+# An example using the Project MAGNET dataset
+#
 data(magnet)
 tapinit <- 15 # tapers
 dt <- 1 # 1/km
@@ -45,11 +38,6 @@ spp <- spectral_properties(Pspec$taper, db.ci=TRUE)
 spa <- spectral_properties(Aspec$taper, db.ci=TRUE)
 
 # function to create polygon data, and create them
-create_poly <- function(x, y, dy){
-  xx <- c(x, rev(x))
-  yy <- c(y+dy, rev(y-dy))
-  return(data.frame(xx=xx, yy=yy))
-}
 pspp <- create_poly(Pspec$freq, dB(Pspec$spec), spp$stderr.chi.approx)
 psppu <- create_poly(Pspec$freq, dB(Pspec$spec), spp$stderr.chi.upper)
 pspa <- create_poly(Aspec$freq, dB(Aspec$spec), spa$stderr.chi.approx)
