@@ -1,7 +1,7 @@
 
 ##
 
-context("DFT calculations -- stats::fft and fftw::FFT")
+context("DFT calculations -- stats::fft [and fftw::FFT]")
 
 library(stats)
 has.fftw <- require(fftw)
@@ -22,10 +22,6 @@ if (has.fftw){
     expect_is(FFT(xn.), 'complex')
   })
   
-  test_that("fftw::FFT expects numeric or complex", {
-    expect_error(FFT(x.))
-  })
-  
   test_that("stats::fft and fftw::FFT return equivalent results", {
     
     # Forward transform
